@@ -1,10 +1,13 @@
-from pyg_base import df_reindex, df_concat, is_nums, Dict, mul_, loop, dictable, try_back
-from pyg_timeseries._rolling import na2v, ffill, v2na
+from pyg_base import df_reindex, df_concat, is_nums, Dict, mul_, loop, dictable, try_back, is_df, pd2np
+from pyg_timeseries._rolling import na2v, ffill, v2na, fnna
 from pyg_timeseries._ewm import ewma, ewmcorr, ewmstd
+from pyg_timeseries._pandas import fnna_like
 import numpy as np
 import pandas as pd
 
-def _ewmcombine(a, w, n = 1024, vol_days = None, full = False):
+    
+@pd2np
+def _ewmcombine(a, w, n = 128, vol_days = None, corr = None):
     """
     We assume all the joining together etc. is done
     
@@ -15,6 +18,9 @@ def _ewmcombine(a, w, n = 1024, vol_days = None, full = False):
     w : np.ndarray
         2-dimensional array of weights
 
+    n : int
+        days for which we smooth the correlation estimate
+        
     Returns
     -------
     
@@ -62,22 +68,31 @@ def _ewmcombine(a, w, n = 1024, vol_days = None, full = False):
     E(X^2) = w^T * C * w
 
     """
+    fs = fnna(a)
     a_ = na2v(ffill(a))
     w_ = na2v(ffill(w))
+    for i, j in enumerate(fs):
+        w_[:j, i] = 0
     if vol_days:
         vols = v2na(ffill(ewmstd(a, vol_days)))
         a_ = a_ / vols
     else:
         vols = 1
     x = np.sum(a_ * w_, axis = 1)
-    x2 = x**2
     w1 = np.sum(w_, axis = 1)
+    x2 = x**2
     w2 = np.sum(w_ ** 2, axis = 1)
     wij = w1 ** 2 - w2
+    wij[wij == 0] = np.nan
     rho = (x2 - w2)/wij
     erho = ewma(rho, n) # this gives us our estimate for rho at time t
+    if corr is True: 
+        corr = n
+    elif corr is False:
+        corr = None
+    full = corr is not None
     if full:
-        c = ewmcorr(a, n)
+        c = ewmcorr(a, corr) if isinstance(corr, int) else corr
         c_ = np.zeros(c.shape)
         variance = np.empty(c.shape[0])
         cip = np.zeros(c.shape[1:])
@@ -94,7 +109,8 @@ def _ewmcombine(a, w, n = 1024, vol_days = None, full = False):
     vol = ffill(np.sqrt(variance))
     mult = 1/vol
     data = x/vol
-    return dict(data = data, vol = vol, rho = erho, mult = mult, cor = c_ if full else None, vols = vols if vol_days else 1)
+    normalized_mult = mult * w1
+    return dict(data = data, vol = vol, rho = erho, mult = mult, normalized_mult = normalized_mult, cor = c_ if full else None, vols = vols if vol_days else 1)
 
 
 @loop(list)
@@ -107,7 +123,7 @@ def _col(value, i):
         return value        
     
 
-def ewmcombine(tss, wgts, n = 1024, full = False, join = 'oj', method = None):
+def ewmcombine(tss, wgts, n = 128, vol_days = None, corr = None, join = 'oj', method = None):
     """
     We expect tss and wgts to be provided as lists at the moment
 
@@ -115,16 +131,19 @@ def ewmcombine(tss, wgts, n = 1024, full = False, join = 'oj', method = None):
     ----------
     tss : list of timeseries
         the data we want to combine into a single signal
-    wgts : list of weights, either floats or timeseries
+    wgts : list/dataframe of weights, either floats or timeseries
         
-    n : TYPE, optional
-        DESCRIPTION. The default is 1024.
-    full : TYPE, optional
-        DESCRIPTION. The default is False.
-    join : TYPE, optional
-        DESCRIPTION. The default is 'oj'.
+    n : int, optional
+        Length of data for correlation calculation. 
+    corr : a 3-d numpy matrix. optional
+        The output from ewmcorr. If provided, a full calculation of correlation is used
+    vol_days:
+        The input assumes that tss are broadly N(0,1) if this is false, the correlation calculation will be off. 
+        You can set vol_days and we will normalize each tss on the fly. If so, the normalized data will be available in vols output
+    join : str, optional
+        how we join the tss to a single frame. The default is 'oj' for outer join.
     method : TYPE, optional
-        DESCRIPTION. The default is None.
+        how we fill nans when we join the tss to a single frame. The default is None .
 
     Returns
     -------
@@ -132,31 +151,36 @@ def ewmcombine(tss, wgts, n = 1024, full = False, join = 'oj', method = None):
         DESCRIPTION.
 
     """
-    cols = [tuple(ts.columns) for ts in tss if len(ts.shape)==2 and ts.shape[1] > 1]
+    if is_df(tss):
+        wgts = fnna_like(tss, wgts).ffill()
+        return _ewmcombine(tss, wgts, n = n, vol_days = vol_days, corr = corr)      
+    if len(tss) == 1:
+        return Dict(data = tss[0], vol = None, rho = None, mult = 1/wgts[0], normalized_mult = 1, cor = None, vols = 1)            
+    cols = [tuple(ts.columns) for ts in tss if is_df(ts) and ts.shape[1] > 1]
     if len(cols):
         assert len(set(cols)) == 1
         cols = list(cols[0])
-        res = Dict(dictable([ewmcombine(_col(tss, i), _col(wgts, i), n = n, full = full, join = join, method = method) for i in range(len(cols))]))
+        res = Dict(dictable([ewmcombine(_col(tss, i), _col(wgts, i), n = n, corr = corr, join = join, method = method) for i in range(len(cols))]))
         res = res.do(try_back(lambda v: df_concat(v, cols)))
         return res        
     if is_nums(wgts):
         tss, wgts = zip(*[(ts if w>0 else -ts, abs(w)) for ts, w in zip(tss, wgts) if w!=0]) ## we drop any zero weights and invert any negative weights
         a = df_concat(list(tss))
-        w = np.array([list(wgts)] * len(a))
+        w = fnna_like(a, wgts, 0.).values.astype(float)
     else:
         tss, wgts = zip(*[(mul_(ts,np.sign(df_reindex(w, ts, 'ffill'))), abs(w)) for ts, w in zip(tss, wgts)]) ## if we is a timeseries, we multiply the two after reindexing w to ts
         a = df_concat(list(tss))
-        w = df_concat([df_reindex(wgt, a.index, method = 'ffill') for wgt in wgts]).values
+        w = df_concat([df_reindex(wgt, a.index, method = 'ffill') for wgt in wgts]).values.astype(float)
     if len(tss) == 1:
-        return Dict(data = tss[0], vol = None, rho = None, mult = None, cor = None, vols = None)            
+        return Dict(data = tss[0], vol = None, rho = None, mult = 1/wgts[0], normalized_mult = 1, cor = None, vols = 1)            
     idx = a.index
-    res = _ewmcombine(a.values, w, n = 1024, vol_days = 120, full = False)
+    res = _ewmcombine(a.values, w, n = n, vol_days = vol_days, corr = corr)
     res = Dict({k : (pd.Series(v, idx) if len(v.shape) == 1 else [pd.Series(v[:,i], idx) for i in range(v.shape[1])]) if isinstance(v, np.ndarray) and v.shape[0] == len(idx) else v for k, v in res.items()})
     return res
 
-ewmcombine.output = ['data', 'vol', 'rho', 'mult', 'cor', 'vols']
+ewmcombine.output = ['data', 'vol', 'rho', 'mult', 'normalized_mult',  'cor', 'vols']
 
-def ewmcombined(tss, wgts, n = 1024, full = False, join = 'oj', method = None):
+def ewmcombined(tss, wgts, n = 128, vol_days = None, corr = False, join = 'oj', method = None):
     """
     We expect tss and wgts to be provided as lists at the moment
 
@@ -164,16 +188,19 @@ def ewmcombined(tss, wgts, n = 1024, full = False, join = 'oj', method = None):
     ----------
     tss : list of timeseries
         the data we want to combine into a single signal
-    wgts : list of weights, either floats or timeseries
+    wgts : list/dataframe of weights, either floats or timeseries
         
-    n : TYPE, optional
-        DESCRIPTION. The default is 1024.
-    full : TYPE, optional
-        DESCRIPTION. The default is False.
-    join : TYPE, optional
-        DESCRIPTION. The default is 'oj'.
+    n : int, optional
+        Length of data for correlation calculation. 
+    corr : a 3-d numpy matrix. optional
+        The output from ewmcorr. If provided, a full calculation of correlation is used
+    vol_days:
+        The input assumes that tss are broadly N(0,1) if this is false, the correlation calculation will be off. 
+        You can set vol_days and we will normalize each tss on the fly. If so, the normalized data will be available in vols output
+    join : str, optional
+        how we join the tss to a single frame. The default is 'oj' for outer join.
     method : TYPE, optional
-        DESCRIPTION. The default is None.
+        how we fill nans when we join the tss to a single frame. The default is None .
 
     Returns
     -------
@@ -181,6 +208,6 @@ def ewmcombined(tss, wgts, n = 1024, full = False, join = 'oj', method = None):
         DESCRIPTION.
 
     """
-    return ewmcombine(tss, wgts, n = n, full = full, join = join, method = method)['data']
+    return ewmcombine(tss, wgts, n = n, corr = corr, join = join, method = method)['data']
 
     
\ No newline at end of file
