@@ -11,14 +11,24 @@ __all__ = ['ffill', 'bfill', 'fnna', 'na2v', 'v2na', 'diff', 'shift', 'ratio', '
 ## parameters
 ##
 ###############
-    
+
+
 @loop(list, dict)
-def _vec(vec, n, value = np.nan):
-    if vec is None:
-        vec = np.full(abs(n), value)
-    elif is_num(vec):
-        vec = np.array([vec])
-    return vec.copy()
+def _vec(a, vec, n, value = np.nan, axis = 0):
+    if len(a.shape)==2:
+        shape = a.shape[1] if axis==0 else a.shape[0]
+        if vec is None:
+            return [np.full(abs(n), value)]*shape
+        elif is_num(vec):
+            return [np.array([vec])]*shape
+        else:
+            return vec.copy()
+    else:        
+        if vec is None:
+            return np.full(abs(n), value)
+        elif is_num(vec):
+            return np.array([vec])
+        return vec.copy()
 
 @loop_all
 @pd2np
@@ -34,6 +44,22 @@ def _fnna(a, n):
             i = i+1
             if i == _n:
                 return j
+
+
+@loop_all
+@pd2np
+@compiled
+def _fnnz(a, n=1):
+    if n == 0:
+        raise ValueError('n must be non-zero')
+    i = 0
+    _n = abs(n)
+    s = (0,a.shape[0],1) if n>0 else (a.shape[0]-1,-1,-1)
+    for j in range(*s):
+        if not np.isnan(a[j]) and a[j]!=0.:
+            i = i+1
+            if i == _n:
+                return j
             
 
 @compiled
@@ -153,8 +179,8 @@ def _bfill(a, limit = -1):
 
 @loop_all
 @pd2np
-def _rolling_window(a, window, min_count, func, vec = None):
-    vec = _vec(vec,0)
+def _rolling_window(a, window, min_count, func, vec = None, axis = 0):
+    vec = _vec(a,vec,0,axis=axis)
     mask = ~np.isnan(a)
     na = a[mask]
     n = len(na)
@@ -178,53 +204,95 @@ def _rolling_window(a, window, min_count, func, vec = None):
 @loop_all
 @pd2np
 @compiled
-def _diff(a, n, vec, i):
+def _diff(a, n, vec, i, time, t = np.nan):
+    """
+    a = cumsum(np.array(range(10))) * 1.
+    n = 1
+    vec = np.array([np.nan])
+    i = 0; t = np.nan
+    time = np.array([0,0,1,1,1,2,2,2,2,3])
+
+    """
     vec = vec.copy()
     _n = abs(n)
     s = (0,a.shape[0],1) if n>0 else (a.shape[0]-1,-1,-1)
     res = np.empty_like(a)
+    prev = vec[i]
     for j in range(*s):
         if np.isnan(a[j]):
             res[j] = np.nan
         else:
-            res[j] = a[j] - vec[i]
+            if not (time[j] == t):
+                i = (i+1) % _n
+                prev = vec[i]
+                # print('moving forward in time, new previous is', prev)
+            t = time[j]
+            res[j] = a[j] - prev
             vec[i] = a[j]
-            i = (i+1) % _n
-    return res, vec, i
+            # print('prev is ', prev, 'time is ', t, 'res is', a[j], '-', prev, '=', res[j])
+    return res, vec, i, t
 
 @loop_all
 @pd2np
 @compiled
-def _diff1(a, vec):
+def _diff1(a, vec, time, i = 0, t = np.nan):
     vec = vec.copy()
-    v = vec[0]
+    s = (0,a.shape[0],1)
     res = np.empty_like(a)
-    for j in range(a.shape[0]):
+    prev = vec[i]
+    for j in range(*s):
         if np.isnan(a[j]):
             res[j] = np.nan
         else:
-            res[j] = a[j] - v
-            v = a[j]
-    vec[0] = v
-    return res, vec, 0
+            if not (time[j] == t):
+                prev = vec[i]
+            t = time[j]
+            res[j] = a[j] - prev
+            vec[i] = a[j]
+    return res, vec, i, t
+
+
+@loop_all
+def _tdiff(a, n, vec, i, time = None, t = None):
+    time = clock(a, time, t)
+    t = 0 if t is None or np.isnan(t) else t
+    return _diff(a = a, n = n, vec = vec, i = i, time = time, t = t)
+
+
+@loop_all
+def _tdiff1(a, vec, i = 0, time = None, t = None):
+    time = clock(a, time, t)
+    t = 0 if t is None or np.isnan(t) else t
+    return _diff1(a = a, vec = vec, i = i, time = time, t = t)
 
 
 @loop_all
 @pd2np
 @compiled
-def _ratio(a, n, vec, i):
+def _ratio(a, n, vec, i, time, t = np.nan):
     vec = vec.copy()
     _n = abs(n)
     s = (0,a.shape[0],1) if n>0 else (a.shape[0]-1,-1,-1)
     res = np.empty_like(a)
+    prev = vec[i]
     for j in range(*s):
         if np.isnan(a[j]):
             res[j] = np.nan
         else:
-            res[j] = np.nan if vec[i] == 0 else a[j] / vec[i] 
+            if not (time[j] == t):
+                i = (i+1) % _n
+                prev = vec[i]
+            t = time[j]
+            res[j] = np.nan if prev == 0 else a[j] / prev 
             vec[i] = a[j]
-            i = (i+1) % _n
-    return res, vec, i
+    return res, vec, i, t
+
+@loop_all
+def _tratio(a, n, vec, i, time = None, t = None):
+    time = clock(a, time, t)
+    t = 0 if t is None or np.isnan(t) else t
+    return _ratio(a = a, n = n, vec = vec, i = i, time = time, t = t)
+
 
 @loop_all
 @pd2np
@@ -398,6 +466,28 @@ def fnna(a, n=1, axis = 0):
     """
     return _fnna(a, n, axis = axis)
 
+def fnnz(a, n=1, axis = 0):
+    """
+    returns the index in a of the nth first non-nan and non-zero.
+    
+    :Parameters:
+    ------------
+    a : array/timeseries
+    n: int, optional, default = 1
+
+    :Example:
+    ---------
+    >>> a = np.array([[np.nan,0.,1.,np.nan,np.nan,2,np.nan,np.nan,np.nan], [0.,0.,0.,np.nan,np.nan,2,np.nan,np.nan,np.nan]]).T
+    >>> #                        ^                                             ^                    ^
+    >>> #                      2nd pos                                 zeros dont count      first non-zero
+
+    >>> assert list(fnnz(a)) == [2,5] 
+    
+    """
+    return _fnnz(a, n, axis = axis)
+
+
+
 def bfill(a, n = -1, axis = 0):
     """
     equivalent to a.fillna('bfill'). There is no state-aware as this function is forward looking
@@ -535,7 +625,7 @@ def init2v(a, n = 0, new = np.nan):
     return _init2v(a, n, new)
 
 
-def diff(a, n=1, axis = 0, data = None, state = None):
+def diff(a, n=1, time = None, axis = 0, data = None, state = None):
     """
     equivalent to a.diff(n) in pandas if there are no nans. If there are, we SKIP nans rather than propagate them.
 
@@ -566,11 +656,12 @@ def diff(a, n=1, axis = 0, data = None, state = None):
     """
     if n == 0:
         return a - a
-    state = state or Dict(vec = None, i = 0)
-    state.vec = _vec(state.vec, n)
-    return first_(_diff1(a, vec = state.vec, axis = axis) if n == 1 else _diff(a, n, axis = axis, **state))
+    state = state or Dict(vec = None, i = 0, t = np.nan)
+    state.vec = _vec(a,state.vec, n, axis=axis)
+    return first_(_tdiff1(a, time = time, axis = axis, **state) if n == 1 
+                  else _tdiff(a, n = n, time = time, axis = axis, **state))
 
-def diff_(a, n=1, axis = 0, data = None, instate = None):
+def diff_(a, n=1, time = None, axis = 0, data = None, instate = None):
     """
     returns a forward filled array, up to n values forward. 
     Equivalent to diff(a,n) but returns the full state. See diff for full details
@@ -578,9 +669,11 @@ def diff_(a, n=1, axis = 0, data = None, instate = None):
     """
     if n == 0:
         return Dict(data = a - a, state = instate)
-    state = instate or Dict(vec = None, i = 0) 
-    state.vec = _vec(state['vec'], n)
-    return _data_state(['data', 'vec', 'i'], _diff1(a, state.vec, axis = axis) if n == 1 else _diff(a, n, axis = axis, **state))
+    state = instate or Dict(vec = None, i = 0, t = np.nan) 
+    state.vec = _vec(a, state['vec'], n, axis=axis)
+    return _data_state(['data', 'vec', 'i', 't'], 
+                       _tdiff1(a, time = time, axis = axis, **state) if n == 1 
+                       else _tdiff(a, n=n, time = time, axis = axis, **state))
 
 diff_.output = ['data', 'state']
         
@@ -628,7 +721,7 @@ def shift(a, n=1, axis = 0, data = None, state = None):
     if n == 0:
         return a
     state = state or Dict(vec = None, i = 0,)
-    state.vec = _vec(state.vec, n)
+    state.vec = _vec(a, state.vec, n, axis=axis)
     return first_(_shift1(a, state.vec, axis = axis) if n == 1 else _shift(a, n, axis = axis, **state))
 
 def shift_(a, n=1, axis = 0, instate = None):
@@ -639,12 +732,12 @@ def shift_(a, n=1, axis = 0, instate = None):
     if n == 0:
         return Dict(data = a, state = instate)
     state = instate or Dict(vec = None, i = 0,)
-    state.vec = _vec(state.vec, n)
+    state.vec = _vec(a, state.vec, n, axis=axis)
     return _data_state(['data', 'vec', 'i'], _shift1(a, vec = state.vec, axis = axis) if n == 1 else _shift(a, n, axis = axis, **state))
 
 shift_.output = ['data', 'state']
         
-def ratio(a, n=1, data = None, state = None):
+def ratio(a, n=1, time = None, data = None, state = None, axis = 0):
     """
     Equivalent to a.diff() but in log-space..
     
@@ -667,14 +760,14 @@ def ratio(a, n=1, data = None, state = None):
     >>> assert eq(ratio(a), pd.Series([np.nan, 2, 1.5, 4/3,1.25], drange(-4)))
     >>> assert eq(ratio(a,2), pd.Series([np.nan, np.nan, 3, 2, 5/3], drange(-4)))
     """
-    state = state or Dict(vec = None, i = 0)
-    state.vec = _vec(state.vec, n)
-    return first_(_ratio(a, n, **state))
+    state = state or Dict(vec = None, i = 0, t = np.nan)
+    state.vec = _vec(a, state.vec, n, axis=axis)
+    return first_(_tratio(a, n, axis = axis, time = time, **state))
 
-def ratio_(a, n=1, data = None, instate = None):
-    state = instate or Dict(vec = None, i = 0) 
-    state.vec = _vec(state.vec, n)
-    return _data_state(['data', 'vec', 'i'], _ratio(a, n, **state))
+def ratio_(a, n=1, time = None, axis = 0, data = None, instate = None):
+    state = instate or Dict(vec = None, i = 0, t = np.nan) 
+    state.vec = _vec(a, state.vec, n, axis=axis)
+    return _data_state(['data', 'vec', 'i', 't'], _tratio(a, n, time = time, **state))
 
 ratio_.output = ['data', 'state']
 
@@ -749,7 +842,7 @@ def rolling_mean(a, n, time = None, axis = 0, data = None, state = None):
 
     """
     state = state or Dict(t0 = 0, t1 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return first_(_trolling_mean(a, n, time = time, denom = n, axis = axis, **state))
 
 def rolling_rms(a, n, time = None, axis = 0, data = None, state = None):
@@ -822,7 +915,7 @@ def rolling_rms(a, n, time = None, axis = 0, data = None, state = None):
 
     """
     state = state or Dict(t0 = 0, t2 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return first_(_trolling_rms(a, n, time = time, denom = n, axis = axis, **state))
 
 def rolling_sum(a, n, time = None, axis = 0, data = None, state = None):
@@ -893,7 +986,7 @@ def rolling_sum(a, n, time = None, axis = 0, data = None, state = None):
     ## The sum is then calculated from last observation in day 0 (i.e. 2) and then 3. and then with 4. since these are again, from same day
     """
     state = state or Dict(t0 = 0, t1 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return first_(_trolling_mean(a, n, time = time, denom = 1, axis = axis, **state))
 
 def rolling_std(a, n, time = None, axis = 0, data = None, state = None):
@@ -964,7 +1057,7 @@ def rolling_std(a, n, time = None, axis = 0, data = None, state = None):
     ## The sum is then calculated from last observation in day 0 (i.e. 2) and then 3. and then with 4. since these are again, from same day
     """    
     state = state or Dict(t0 = 0, t1 = 0, t2 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return first_(_trolling_std(a, n, time = time, denom = n, axis = axis, **state))
 
 def rolling_skew(a, n, bias = False, time = None, axis = 0, data = None, state = None):
@@ -1027,7 +1120,7 @@ def rolling_skew(a, n, bias = False, time = None, axis = 0, data = None, state =
     >>> assert eq(rolling_skew([a,a**2],10), [rolling_skew(a,10), rolling_skew(a**2,10)])
     """
     state = state or Dict(t0 = 0, t1 = 0, t2 = 0., t3 = 0, vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return first_(_trolling_skew(a, n, time = time, bias = bias, denom = n, axis = axis, **state))
 
 
@@ -1037,7 +1130,7 @@ def rolling_mean_(a, n, time = None, axis = 0, data = None, instate = None):
     For full documentation, look at rolling_mean.__doc__
     """
     state = instate or Dict(t0 = 0, t1 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return _data_state(['data','t0','t1', 'vec','i', 't'],_trolling_mean(a, n, time = time, denom = n, axis = axis, **state))
 
 rolling_mean_.output = ['data','state']
@@ -1048,7 +1141,7 @@ def rolling_rms_(a, n, time = None, axis = 0, data = None, instate = None):
     For full documentation, look at rolling_rms.__doc__
     """
     state = instate or Dict(t0 = 0, t2 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return _data_state(['data','t0','t2', 'vec','i', 't'],_trolling_rms(a, n, time = time, denom = n, axis = axis, **state))
 
 rolling_rms_.output = ['data','state']
@@ -1059,7 +1152,7 @@ def rolling_sum_(a, n, time = None, axis = 0, data = None, instate = None):
     For full documentation, look at rolling_sum.__doc__
     """
     state = instate or Dict(t0 = 0, t1 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return _data_state(['data','t0','t1', 'vec','i', 't'], _trolling_mean(a, n, denom = 1, axis = axis, **state))
 
 rolling_sum_.output = ['data','state']
@@ -1070,7 +1163,7 @@ def rolling_std_(a, n, time = None, axis = 0, data = None, instate = None):
     For full documentation, look at rolling_std.__doc__
     """
     state = instate or Dict(t0 = 0, t1 = 0, t2 = 0., vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return _data_state(['data','t0', 't1', 't2', 'vec', 'i', 't'],_trolling_std(a, n, time = time, denom = n, axis = axis, **state))
 
 
@@ -1082,7 +1175,7 @@ def rolling_skew_(a, n, time = None, bias = False, axis = 0, data = None, instat
     For full documentation, look at rolling_skew.__doc__
     """
     state = instate or Dict(t0 = 0, t1 = 0, t2 = 0., t3 = 0, vec = None, i = 0, t = np.nan)
-    state.vec = _vec(state.vec, n, 0.)
+    state.vec = _vec(a, state.vec, n, 0., axis=axis)
     return _data_state(['data','t0', 't1', 't2', 't3', 'vec','i', 't'], _trolling_skew(a, n, time = time, bias = bias, denom = n, axis = axis, **state))
 
 rolling_skew_.output = ['data','state']
