@@ -1,8 +1,10 @@
 import numpy as np
+import pandas as pd
 from pyg_timeseries._math import stdev_calculation, skew_calculation, cor_calculation
 from pyg_timeseries._decorators import compiled, first_, _data_state
 from pyg_timeseries._rolling import _vec
-from pyg_base import pd2np, loop_all
+from pyg_base import pd2np, loop_all, is_num, is_nums, as_list, is_ts, is_pd, logger
+
 
 __all__ = ['ts_std', 'ts_mean', 'ts_skew', 'ts_count', 'ts_min', 'ts_max', 'ts_rms', 'ts_median',  'ts_sum',
            'ts_std_', 'ts_mean_', 'ts_skew_', 'ts_count_', 'ts_min_', 'ts_max_', 'ts_rms_', 'ts_sum_']
@@ -199,6 +201,8 @@ def ts_sum(a, axis = 0, data = None, state = None):
     >>> assert new == ts_sum(a)
     """
     state = state or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return first_(_ts_sum(a, axis = axis, **state))
 
 def ts_sum_(a, axis = 0, data = None, instate = None):
@@ -207,6 +211,8 @@ def ts_sum_(a, axis = 0, data = None, instate = None):
     See ts_sum for full documentation        
     """
     state = instate or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return _zip(_ts_sum(a, axis = axis, **state))
 
 @loop_all
@@ -256,6 +262,8 @@ def ts_mean(a, axis = 0, data = None, state = None):
     >>> assert new == ts_mean(a)
     """
     state = state or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return first_(_ts_mean(a, axis = axis, **state))
 
 def ts_mean_(a, axis = 0, data = None, instate = None):
@@ -264,6 +272,9 @@ def ts_mean_(a, axis = 0, data = None, instate = None):
     See ts_mean for full documentation        
     """
     state = instate or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+
+        a = np.array(as_list(a))
     return _zip(_ts_mean(a, axis = axis, **state))
 
 
@@ -329,6 +340,10 @@ def ts_cor_(a, b, min_sample = 3, axis = 0, data = None, instate = None):
     See ts_std for full documentation    
     """
     state = instate or dict(vec = _vec(None,6,0.))
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
+    if is_num(b) or (isinstance(b, list) and is_nums(b)):
+        b = np.ndarray(as_list(b))
     return _zip(_ts_cor(a, b, min_sample, axis = axis, **state))
 
 
@@ -367,6 +382,8 @@ def ts_std(a, axis = 0, data = None, state = None):
 
     """
     state = state or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return first_(_ts_std(a, axis = axis, **state))
 
 def ts_std_(a, axis = 0, data = None, instate = None):
@@ -375,6 +392,8 @@ def ts_std_(a, axis = 0, data = None, instate = None):
     See ts_std for full documentation    
     """
     state = instate or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return _zip(_ts_std(a, axis = axis, **state))
 
 @loop_all
@@ -384,6 +403,7 @@ def _ts_rms(a, vec = None):
     return np.sqrt(vec[2]/vec[0]), vec
     
 
+
 def ts_rms(a, axis = 0, data = None, state = None):
     """
     ts_rms(a) is equivalent to (a**2).mean()**0.5
@@ -426,6 +446,8 @@ def ts_rms(a, axis = 0, data = None, state = None):
 
     """
     state = state or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return first_(_ts_rms(a, axis = axis, **state))
 
 def ts_rms_(a, axis = 0, data = None, instate = None):
@@ -434,6 +456,8 @@ def ts_rms_(a, axis = 0, data = None, instate = None):
     see ts_rms for full documentation
     """
     state = instate or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return _zip(_ts_rms(a, axis = axis, **state))
 
 
@@ -489,6 +513,8 @@ def ts_skew(a, bias = False, min_sample = 0.25, axis = 0, data = None, state = N
     >>> assert new == ts_skew(a)
     """
     state = state or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return first_(_ts_skew(a, bias = bias, min_sample = min_sample, axis = axis, **state))
     
 def ts_skew_(a, bias = False, min_sample = 0.25, axis = 0, data = None, instate = None):
@@ -497,6 +523,8 @@ def ts_skew_(a, bias = False, min_sample = 0.25, axis = 0, data = None, instate
     See ts_skew for full details
     """
     state = instate or {}
+    if is_num(a) or (isinstance(a, list) and is_nums(a)):
+        a = np.array(as_list(a))
     return _zip(_ts_skew(a, bias = bias, min_sample = min_sample, axis = axis, **state))
 
 
@@ -512,3 +540,35 @@ ts_std_.output = ['data', 'state']
 ts_skew_.output = ['data', 'state']
 
 
+def ts_interval(ts, min_freq = 0.5):
+    """
+    returns the most common time difference in the timeseries
+
+    :Example:
+    ----------
+    >>> ts = pd.Series(range(100), drange(-200,0,'1b')[:100])
+    >>> assert ts_interval(ts) == datetime.timedelta(1)
+
+    >>> ts = pd.Series(range(100), drange(-1,0,'5n')[:100])
+    >>> assert ts_interval(ts) ==  datetime.timedelta(minutes = 5)
+    
+    >>> ts = pd.Series(range(10000), range(10000))
+    >>> ts = ts[ts % 2 != 0]
+    >>> ts = ts[ts % 3 != 0]
+    >>> ts = ts[ts % 5 != 0]
+    >>> assert ts_interval(ts) == 4
+    
+    
+    """
+    if len(ts)<2:
+        return None
+    v = ts.index if is_pd(ts) else ts
+    intervals = pd.Series(v, v).diff().iloc[1:]
+    res = intervals.mode()[0]
+    freq = len(intervals[intervals.values==res]) / len(intervals)
+    if freq>=min_freq:
+        return res
+    else: 
+        logger.warning('timeseries is irregular, with mode frequency less than %1.2f. returning median interval instead'%freq)
+        return intervals.median()
+    
\ No newline at end of file
